/*
 * Copyright (C) 2002 Roman Zippel <zippel@linux-m68k.org>
 * Released under the terms of the GNU GPL v2.0.
 */

#include <qlistview.h>

class ConfigLineEdit;
class ConfigItem;

enum colIdx {
	promptColIdx, nameColIdx, noColIdx, modColIdx, yesColIdx, dataColIdx, colNr
};

class ConfigList : public QListView {
	Q_OBJECT
	typedef class QListView Parent;
public:
	ConfigList(QWidget *p);
	void reinit(void);

	ConfigLineEdit *lineEdit;

	bool showAll(void) const
	{
		return _showAll;
	}
	bool showName(void) const
	{
		return _showName;
	}
	bool showRange(void) const
	{
		return _showRange;
	}
	bool showData(void) const
	{
		return _showData;
	}
	bool showFull(void) const
	{
		return _showFull;
	}
protected:
	void keyPressEvent(QKeyEvent *e);
	void contentsMousePressEvent(QMouseEvent *e);
	void contentsMouseReleaseEvent(QMouseEvent *e);
	void contentsMouseMoveEvent(QMouseEvent *e);
	void contentsMouseDoubleClickEvent(QMouseEvent *e);
public slots:
	void setShowAll(bool b);
	void setShowName(bool b);
	void setShowRange(bool b);
	void setShowData(bool b);
	void setShowFull(bool b);

	void updateList(ConfigItem *item);
	void setValue(ConfigItem* item, tristate val);
	void changeValue(ConfigItem* item);
signals:
	void showAllChanged(bool showAll);
	void showNameChanged(bool showName);
	void showRangeChanged(bool showRange);
	void showDataChanged(bool showData);
	void showFullChanged(bool showFull);

public:
	void updateListAll(void)
	{
		updateAll = true;
		updateList(NULL);
		updateAll = false;
	}
public:
	int mapIdx(colIdx idx)
	{
		return colMap[idx];
	}
	void addColumn(colIdx idx, const QString& label)
	{
		colMap[idx] = Parent::addColumn(label);
		colRevMap[colMap[idx]] = idx;
	}
	void removeColumn(colIdx idx)
	{
		int col = colMap[idx];
		if (col >= 0) {
			Parent::removeColumn(col);
			colRevMap[col] = colMap[idx] = -1;
		}
	}
	bool setRootMenu(struct menu *menu);

	bool updateAll;

	QPixmap symbolYesPix, symbolModPix, symbolNoPix;
	QPixmap choiceYesPix, choiceNoPix, menuPix, menuInvPix;
private:
	bool _showAll, _showName, _showRange, _showData, _showFull;

	int colMap[colNr];
	int colRevMap[colNr];

	struct menu *rootEntry;
};

class ConfigItem : public QListViewItem {
	typedef class QListViewItem Parent;
public:
	ConfigItem(QListView *parent, ConfigItem *after, struct menu *m)
	: Parent(parent, after), menu(m)
	{
		init();
	}
	ConfigItem(ConfigItem *parent, ConfigItem *after, struct menu *m)
	: Parent(parent, after), menu(m)
	{
		init();
	}
	~ConfigItem(void);
	void init(void);
#if QT_VERSION >= 300
	void okRename(int col);
#endif
	void updateMenu(void);
	ConfigList* listView() const
	{
		return (ConfigList*)Parent::listView();
	}
	void setText(colIdx idx, const QString& text)
	{
		Parent::setText(listView()->mapIdx(idx), text);
	}
	QString text(colIdx idx) const
	{
		return Parent::text(listView()->mapIdx(idx));
	}
	void setPixmap(colIdx idx, const QPixmap& pm)
	{
		Parent::setPixmap(listView()->mapIdx(idx), pm);
	}
	const QPixmap* pixmap(colIdx idx) const
	{
		return Parent::pixmap(listView()->mapIdx(idx));
	}

	struct menu *menu;
#if QT_VERSION < 300
	void setEnabled(bool b)
	{
		enabled = b;
	}
	bool enabled;
#endif
};

class ConfigLineEdit : public QLineEdit {
	Q_OBJECT
	typedef class QLineEdit Parent;
public:
	ConfigLineEdit(QWidget * parent)
	: QLineEdit(parent)
	{ }
	void show(ConfigItem *i);
	void keyPressEvent(QKeyEvent *e);
signals:
	void lineChanged(ConfigItem *item);

public:
	ConfigItem *item;
};

class ConfigView : public QMainWindow {
	Q_OBJECT
public:
	ConfigView(void);
public slots:
	void setHelp(QListViewItem* item);
	void loadConfig(void);
	void saveConfig(void);
	void saveConfigAs(void);

protected:
	void closeEvent(QCloseEvent *e);

	ConfigList *configList;
	QTextView *helpText;
	QToolBar *toolBar;
	int showAllId;
	int showNameId;
	int showRangeId;
	int showDataId;
	int showFullId;
};
