/*
 * Copyright (C) 2002 Roman Zippel <zippel@linux-m68k.org>
 * Released under the terms of the GNU GPL v2.0.
 */

#ifndef ZCONF_H
#define ZCONF_H

#include "expr.h"

#ifdef __cplusplus
extern "C" {
#endif

void symbol_end(char *help);
int zconfparse(void);
void zconfdump(FILE *out);

extern int zconfdebug;
void zconf_starthelp(void);
void zconf_initscan(const char *name);
void zconf_nextfile(const char *name);
int zconf_lineno(void);
char *zconf_curname(void);


void conf_parse(const char *name);
int conf_read(const char *name);
int conf_write(const char *name);

extern const char conf_def_filename[];
extern char conf_filename[128];

/* menu.c */
void menu_init(void);
void menu_add_menu(void);
void menu_end_menu(void);
void menu_add_entry(struct symbol *sym);
void menu_end_entry(void);
struct statement *create_stmt(int token);
void menu_add_dep(struct expr *dep);
struct statement *menu_add_stmt(int token, char *prompt, struct symbol *def, struct expr *dep);
void menu_finalize(struct menu *parent);
void menu_set_type(int type);
bool menu_is_visible(struct menu *menu);
char *menu_get_prompt(struct menu *menu);
struct file *file_lookup(const char *name);
int file_write_dep(const char *name);

extern struct menu rootmenu;
extern struct menu *current_entry;

/* symbol.c */
const char *sym_type_name(int type);
struct symbol *lookup_symbol(char *name, int isconst);
void sym_calc_value(struct symbol *sym);
int contains_symbol(struct expr *dep, struct symbol *sym);
bool depends_on_symbol(struct expr *dep, struct symbol *sym);
bool sym_tristate_within_range(struct symbol *sym, tristate tri);
bool sym_set_tristate_value(struct symbol *sym, tristate tri);
tristate sym_toggle_tristate_value(struct symbol *sym);
bool sym_string_valid(struct symbol *sym, const char *newval);
bool sym_set_string_value(struct symbol *sym, const char *newval);
bool sym_is_changable(struct symbol *sym);
struct statement *sym_get_choice_stmt(struct symbol *sym);
struct statement *sym_get_default_stmt(struct symbol *sym);
const char *sym_get_string_value(struct symbol *sym);

static inline tristate sym_get_tristate_value(struct symbol *sym)
{
	return S_TRI(sym->curr);
}


static inline struct symbol *sym_get_choice_value(struct symbol *sym)
{
	return (struct symbol *)S_VAL(sym->curr);
}

static inline bool sym_set_choice_value(struct symbol *ch, struct symbol *chval)
{
	return sym_set_tristate_value(chval, yes);
}

static inline bool sym_is_choice(struct symbol *sym)
{
	return sym->flags & SYMBOL_CHOICE ? true : false;
}

static inline bool sym_is_choice_value(struct symbol *sym)
{
	return sym->flags & SYMBOL_CHOICEVAL ? true : false;
}

static inline bool sym_is_optional(struct symbol *sym)
{
	return sym->flags & SYMBOL_OPTIONAL ? true : false;
}

static inline bool sym_has_value(struct symbol *sym)
{
	//return S_VAL(sym->def) != NULL;
	return sym->flags & SYMBOL_NEW ? false : true;
}

static inline int sym_get_type(struct symbol *sym)
{
	int type = sym->type;
	if (type == SYMBOL_TRISTATE) {
		if (sym_is_choice_value(sym) && sym->visible == yes)
			type = SYMBOL_BOOLEAN;
		else {
			sym_calc_value(modules_sym);
			if (S_TRI(modules_sym->curr) == no)
				type = SYMBOL_BOOLEAN;
		}
	}
	return type;
}

#ifdef __cplusplus
}
#endif

#endif /* ZCONF_H */
